# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from datetime import datetime

from django import forms
from django.contrib import admin

from .models import Notification, UserNotification


@admin.register(UserNotification)
class UserNotificationAdmin(admin.ModelAdmin):
    actions = ['really_delete_selected']

    list_display = (
        'user',
        'notification'
    )

    search_fields = (
        'user__username',
        'notification__name'
    )

    def get_actions(self, request):
        actions = super(UserNotificationAdmin, self).get_actions(request)
        del actions['delete_selected']
        return actions

    def really_delete_selected(self, request, queryset):
        for obj in queryset:
            obj.delete()

        # The queryset cache has the count after the loop above, no
        # need for extra call or storing count in variable.
        if queryset.count() == 1:
            message_bit = "1 UserNotification entry was"
        else:
            message_bit = "%s usernotification entries were" % queryset.count()
        self.message_user(request, "%s successfully deleted." % message_bit)

    really_delete_selected.short_description = "Delete selected entries"


@admin.register(Notification)
class NotificationAdmin(admin.ModelAdmin):
    readonly_fields = ('management',)

    fields = (
        'name_en',
        'name_is',
        'message_en',
        'message_is',
        'active_from',
        'expires',
        'needs_approval',
        'snooze_time',
        'groups',
        'look',
        'image',
        'management'
    )

    def management(self, instance):
        EXCLUDED_FIELDS = [
            'groups',
            'id',
            'recipients',
            'attachment',
            'active_from',
            'snooze_lock',
            'recipients',
            'send_email'
        ]
        params = ''
        for field in instance.__class__._meta.get_fields():
            if field.name in EXCLUDED_FIELDS:
                continue
            attr = getattr(instance, field.name, None)
            if attr is None or attr == '':
                continue
            if isinstance(attr, datetime):
                attr = attr.isoformat()
            if field.name == 'message':
                attr = ''.join(attr.split('\n'))
                attr = ''.join(attr.split('\r'))
            params += ' --{}=\'{}\''.format(field.name, attr)
        groups = ','.join(map(str, instance.groups.values_list('id', flat=True)))
        if groups:
            params += ' --groups={}'.format(groups)

        return "python manage.py import_notification{}".format(params)

    management.short_description = 'Management'

    def save_model(self, request, obj, form, change):
        super(NotificationAdmin, self).save_model(request, obj, form, change)
        for group in form.cleaned_data['groups']:
            for user in group.user_set.all():
                UserNotification.objects.get_or_create(notification=obj, user=user)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2019-02-20 14:56
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0001_initial'),
    ]


    operations = [
        migrations.AddField(
            model_name='notification',
            name='image',
            field=models.ImageField(
                upload_to='notification_imgs',
                null=True,
                blank=True,
                help_text="Image to accompany notification (optional)",
            )
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.core.management.base import BaseCommand

from accounts.models import UserNotification


class Command(BaseCommand):
    help = 'Updates cache entries for notifications'

    def handle(self, *args, **options):
        print('update_unseen_notification_cache')
        unseen = UserNotification.objects.all()
        for un in unseen:
            un.update_unseen_cache()
        print('Cache entries for {} notifications updated'.format(unseen.count()))



# -*- coding: utf-8 -*-
import datetime
import re

from rest_framework import filters
from rest_framework import mixins, serializers, viewsets
from rest_framework import permissions
from rest_framework.response import Response

from django.db.models import Q
from django_filters.rest_framework import DjangoFilterBackend

from notifications.models import UserNotification, Notification

from .serializers import UserNotificationSerializer, UserNotificationPutSerializer, \
    NotificationSerializer


class UserNotificationViewSet(mixins.RetrieveModelMixin, mixins.UpdateModelMixin, viewsets.GenericViewSet):
    queryset = UserNotification.objects.all()

    permission_classes = (permissions.IsAuthenticated,)

    filter_backends = (filters.SearchFilter, DjangoFilterBackend)

    def get_serializer_class(self):
        if self.request.method in ['PUT', 'PATCH']:
            serializer_class = UserNotificationPutSerializer
        else:
            serializer_class = UserNotificationSerializer
        return serializer_class

    def list(self, request):
        queryset = self.queryset.filter(
            Q(user=self.request.user) &
            (Q(next_display__isnull=True) | Q(next_display__lte=datetime.datetime.now())) &
            (Q(notification__expires__isnull=True) | Q(notification__expires__gte=datetime.datetime.now())) &
            (Q(notification__active_from__isnull=True) | Q(notification__active_from__lt=datetime.datetime.now()))
        ).exclude(answer=True).select_related('notification', 'user').order_by('timestamp', 'id')
        path = request.query_params.get('path', None)
        if path is not None:
            queryset = [
                un for un in queryset
                if re.match(un.notification.display_only_if_url_path_matches_regex, path) is not None
            ]
        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    def update(self, request, *args, **kwargs):
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        if instance.user != request.user:
            raise serializers.ValidationError('User can only change his own usernotifications')
        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)

        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}

        return Response(serializer.data)


class NotificationViewSet(mixins.RetrieveModelMixin, viewsets.GenericViewSet):
    queryset = Notification.objects.all()

    permission_classes = (permissions.IsAdminUser,)
    filter_backends = (filters.SearchFilter, DjangoFilterBackend)

    serializer_class = NotificationSerializer

    def list(self, request):
        queryset = Notification.objects.filter()
        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)
